import { MatchRequest } from '../../domain/dtos/match.request';
import { Role } from '../../domain/types/role.enum';
import { TimingFrequency } from '../../domain/types/timing';
import { IDefaultParams } from '../../domain/types/default-params.type';
import { MatchQuery } from '../../queries/match.query';
import { Algorithm } from '../../domain/types/algorithm.enum';

const defaultParams: IDefaultParams = {
  DEFAULT_IDENTIFIER: 0,
  MARGIN_DURATION: 900,
  VALIDITY_DURATION: 365,
  DEFAULT_TIMEZONE: 'Europe/Paris',
  DEFAULT_SEATS: 3,
  DEFAULT_ALGORITHM_SETTINGS: {
    algorithm: Algorithm.CLASSIC,
    strict: false,
    remoteness: 15000,
    useProportion: true,
    proportion: 0.3,
    useAzimuth: true,
    azimuthMargin: 10,
    maxDetourDistanceRatio: 0.3,
    maxDetourDurationRatio: 0.3,
    georouterType: 'graphhopper',
    georouterUrl: 'http://localhost',
  },
};

const mockGeorouterCreator = {
  create: jest.fn().mockImplementation(),
};

describe('Match query', () => {
  it('should be defined', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery).toBeDefined();
  });

  it('should create a query with excluded identifiers', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.identifier = 125;
    matchRequest.exclusions = [126, 127, 128];
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery.exclusions.length).toBe(4);
  });

  it('should create a query with driver role only', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.driver = true;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery.roles).toEqual([Role.DRIVER]);
  });

  it('should create a query with passenger role only', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.passenger = true;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery.roles).toEqual([Role.PASSENGER]);
  });

  it('should create a query with driver and passenger roles', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.passenger = true;
    matchRequest.driver = true;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery.roles.length).toBe(2);
    expect(matchQuery.roles).toContain(Role.PASSENGER);
    expect(matchQuery.roles).toContain(Role.DRIVER);
  });

  it('should create a query with number of seats modified', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.seatsDriver = 1;
    matchRequest.seatsPassenger = 2;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery.requirement.seatsDriver).toBe(1);
    expect(matchQuery.requirement.seatsPassenger).toBe(2);
  });

  it('should create a query with modified algorithm settings', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.algorithm = Algorithm.CLASSIC;
    matchRequest.strict = true;
    matchRequest.useProportion = true;
    matchRequest.proportion = 0.45;
    matchRequest.useAzimuth = true;
    matchRequest.azimuthMargin = 15;
    matchRequest.remoteness = 20000;
    matchRequest.maxDetourDistanceRatio = 0.41;
    matchRequest.maxDetourDurationRatio = 0.42;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
    );
    expect(matchQuery.algorithmSettings.algorithm).toBe(Algorithm.CLASSIC);
    expect(matchQuery.algorithmSettings.restrict).toBe(
      TimingFrequency.FREQUENCY_PUNCTUAL,
    );
    expect(matchQuery.algorithmSettings.useProportion).toBeTruthy();
    expect(matchQuery.algorithmSettings.proportion).toBe(0.45);
    expect(matchQuery.algorithmSettings.useAzimuth).toBeTruthy();
    expect(matchQuery.algorithmSettings.azimuthMargin).toBe(15);
    expect(matchQuery.algorithmSettings.remoteness).toBe(20000);
    expect(matchQuery.algorithmSettings.maxDetourDistanceRatio).toBe(0.41);
    expect(matchQuery.algorithmSettings.maxDetourDurationRatio).toBe(0.42);
  });
});
